import os
import requests
import json
import time
from tqdm.notebook import tqdm
import hashlib
from PIL import Image
import base64
import random
import subprocess  # Importar subprocess
from registro import *
from moviepy.video.io.VideoFileClip import VideoFileClip

FOLDER_PATH_HED = None
IMAGE_PATH_HED = None
RUTA_VIDEO_HED = None
RATIO_HED = None
PROMPT_HED = None
SEED_INPUT_HED = None
SEED_HED = None
CONTINUE_FRAGMENT_HED = None
PROCESS_MULTIPLE_HED = None

def recortar_video(ruta_video):
    """
    Recorta un video eliminando una duración fija del inicio y del final,
    guardándolo con el mismo nombre y en la misma ruta, reemplazando el archivo original.
    
    Args:
        ruta_video (str): Ruta completa del archivo de video (entrada y salida).
    """
    # Duraciones fijas para recortar
    inicio_recorte = 0.13  # Recortar 13 milésimas de segundo del inicio
    final_recorte = 0.15  # Recortar 15 milésimas de segundo del final

    # Carga el video original
    clip = VideoFileClip(ruta_video)
    
    # Recorta el video
    duracion_total = clip.duration
    nuevo_clip = clip.subclip(inicio_recorte, duracion_total - final_recorte)
    
    # Genera un archivo temporal en la misma ruta
    temp_output_path = "temp_" + os.path.basename(ruta_video)
    nuevo_clip.write_videofile(temp_output_path, codec="libx264", audio_codec="aac")
    
    # Reemplaza el archivo original con el archivo temporal
    os.replace(temp_output_path, ruta_video)
    
    print(f"Video procesado y guardado en {ruta_video}")

def video_to_base64(video_path):
    with open(video_path, "rb") as video_file:
        video_base64 = base64.b64encode(video_file.read()).decode('utf-8')
    return video_base64

def eliminar_proyecto(token, joob_ids):
    url = f"https://www.hedra.com/api/app/v1/app/projects/{joob_ids}"
    headers = {
        "Authorization": f"Bearer {token}",
        "sec-ch-ua": '"Not)A;Brand";v="99", "Google Chrome";v="127", "Chromium";v="127"',
        "sec-ch-ua-platform": '"Windows"',
        "sec-ch-ua-mobile": "?0",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, como Gecko) Chrome/127.0.0.0 Safari/537.36",
        "Content-Type": "application/json",
        "Accept": "*/*",
        "Origin": "https://www.hedra.com",
        "Sec-Fetch-Site": "same-origin",
        "Sec-Fetch-Mode": "cors",
        "Sec-Fetch-Dest": "empty",
        "Referer": "https://www.hedra.com/app/characters",
        "Accept-Language": "es-ES,es;q=0.9",
        "Accept-Encoding": "gzip, deflate",
    }

    response = requests.delete(url, headers=headers)

    # Imprime el código de estado de la respuesta y el contenido de la respuesta
    #print(f"Status Code: {response.status_code}")
    #print(f"Response: {response.text}")


def extract_last_frame(video_path, output_image):
    # Verificar si existe un archivo JPG y eliminarlo
    if os.path.exists(output_image):
        os.remove(output_image)
        print(f'Existing file {output_image} deleted.')

    # Primero obtenemos la duración del video
    result = subprocess.run(
        ["ffprobe", "-v", "error", "-show_entries", "format=duration",
         "-of", "default=noprint_wrappers=1:nokey=1", video_path],
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT
    )

    # Convertimos la duración a segundos
    duration = float(result.stdout)

    # Comando para extraer el fotograma en el último segundo
    command = [
        "ffmpeg",
        "-ss", str(duration - 0.1),  # Nos movemos al último segundo del video
        "-i", video_path,
        "-vframes", "1",             # Extraemos un solo fotograma
        "-q:v", "2",                 # Calidad de la imagen (2 es la mejor calidad)
        output_image
    ]

    # Ejecutar el comando
    subprocess.run(command)

    # Estirar la imagen 2 píxeles en altura
    with Image.open(output_image) as img:
        width, height = img.size
        new_height = height + 2
        resized_img = img.resize((width, new_height))
        resized_img.save(output_image)

    print(f"Last frame saved and stretched as {output_image}")





# Función para actualizar la barra de progreso y descargar el video si el progreso es 100%
def update_progress_bar(token, current_progress, file_name_without_ext, joob_ids, ruta_videos, ruta_ava_img, continue_fragment, process_multiple):
    total_steps = 100
    video_url = None
    while current_progress < 1.0:
        # Simula obtener el progreso actualizado
        response = obtener_avatar_sin_cookies(token)
        #print("Proceso:", response)
        project = response['projects'][0]
        current_progress = project['progress']
        video_url = project['videoUrl']  # Obtener la URL del video
        #print("Video URL:", video_url)
        
        # Calcular el contador de progreso
        step = int(current_progress * total_steps)

        os.environ["PROGRESS_HEDRA"] = str(step)

        # Imprimir el progreso en la misma línea
        print(f"\rProgreso: {step}%", end='', flush=True)
        
        time.sleep(2)  # Ajusta el intervalo de actualización aquí

    if video_url:  # Si se ha obtenido una URL del video, descarga el video
        download_video(token, video_url, file_name_without_ext, joob_ids, ruta_videos, ruta_ava_img, continue_fragment, process_multiple)


# Función para generar un nombre de archivo seguro a partir de una URL
def generate_safe_filename(url):
    # Usa un hash MD5 de la URL para generar un nombre de archivo único y seguro
    hash_object = hashlib.md5(url.encode())
    return hash_object.hexdigest() + '.mp4'

# Función para descargar el video desde una URL
def download_video(token, url, file_name_without_ext, joob_ids, ruta_videos, ruta_ava_img, continue_fragment, process_multiple):
    response = requests.get(url, stream=True)
    if response.status_code == 200:
        filename = generate_safe_filename(url)
        with open(f"{ruta_videos}{file_name_without_ext}.mp4", 'wb') as file:
        #with open(f"/tmp/videos/{file_name_without_ext}.mp4", 'wb') as file:
            for chunk in response.iter_content(chunk_size=8192):
                if chunk:
                    file.write(chunk)
        print(f" Video downloaded as {ruta_videos}{file_name_without_ext}.mp4")
        # Ejemplo de uso
        video_path = f"{ruta_videos}{file_name_without_ext}.mp4"

        # Reemplaza con la ruta de tu archivo

        files_path = f"/tmp/audios/{file_name_without_ext}.mp3"
        if os.path.exists(files_path):
            os.remove(files_path)

        output_image = ruta_ava_img #"/tmp/avatar_img.jpg"
        if continue_fragment and not process_multiple:
        #if continue_fragment:
           extract_last_frame(video_path, output_image)
        # Ejecutar la función
        eliminar_proyecto(token, joob_ids)

        recortar_video(f"{ruta_videos}{file_name_without_ext}.mp4")

        os.environ["VIDEO_PATH_HEDRA"] = f"{ruta_videos}{file_name_without_ext}.mp4"

    else:
        print("Error downloading video")

def obtener_avatar_sin_cookies(token):
    url = "https://www.hedra.com/api/app/v1/app/projects"
    headers = {
        "Authorization": f"Bearer {token}",
        "sec-ch-ua": '"Not)A;Brand";v="99", "Google Chrome";v="127", "Chromium";v="127"',
        "sec-ch-ua-platform": '"Windows"',
        "sec-ch-ua-mobile": "?0",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, como Gecko) Chrome/127.0.0.0 Safari/537.36",
        "Content-Type": "application/json",
        "Accept": "*/*",
        "Sec-Fetch-Site": "same-origin",
        "Sec-Fetch-Mode": "cors",
        "Sec-Fetch-Dest": "empty",
        "Referer": "https://www.hedra.com/app/characters",
        "Accept-Language": "es-ES,es;q=0.9",
        "Accept-Encoding": "gzip, deflate",
    }

    response = requests.get(url, headers=headers)
    return response.json()

# Convertir la imagen a base64
def image_to_base64(image_path):
    with open(image_path, "rb") as image_file:
        encoded_string = base64.b64encode(image_file.read()).decode('utf-8')
    return encoded_string


def generar_avatar2(
    authorization_token,  # Token de autorización que puede cambiar
    avatar_image,         # Imagen base64 del avatar
    aspect_ratio="16:9",  # Relación de aspecto, editable
    prompt="",            # Descripción del prompt, editable
    seed_input=True,      # Si es True, puedes proporcionar un seed; si es False, el seed es aleatorio
    seed=None,            # Semilla proporcionada si seed_input es True, sino se genera aleatoriamente
    voice_url="",         # URL del archivo de voz editable
):

    # Si seed_input es False, generamos una semilla aleatoria
    if not seed_input:
        seed = random.randint(1000000, 9999999)  # Genera un número aleatorio de 7 dígitos

    # Definir la URL de la API
    url = "https://www.hedra.com/api/app/v1/app/avatars/predict-async"

    # Cabeceras HTTP
    headers = {
        "Host": "www.hedra.com",
        "Connection": "keep-alive",
        "Authorization": f"Bearer {authorization_token}",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/131.0.0.0 Safari/537.36",
        "sec-ch-ua": "\"Google Chrome\";v=\"131\", \"Chromium\";v=\"131\", \"Not_A Brand\";v=\"24\"",
        "Content-Type": "application/json",
        "Accept": "*/*",
        "Origin": "https://www.hedra.com",
        "Accept-Language": "es-ES,es;q=0.9",
        "Accept-Encoding": "gzip, deflate",
    }

    # Crear el cuerpo de la solicitud
    data = {
        "text": "",
        "avatar_image": avatar_image,
        "aspect_ratio": aspect_ratio,
        "avatar_image_input": {
            "prompt": prompt,
            "seed": seed
        },
        "audio_source": "audio",  # Usando TTS como la fuente de audio
        "voice_url": voice_url
    }

    # Realizar la solicitud POST
    response = requests.post(url, headers=headers, data=json.dumps(data))

    # Verificar la respuesta
    if response.status_code == 200:
        print("Solicitud enviada con éxito.")
        responses = response.json()

        # Extraer el job_id
        job_id = responses['job_id']
        return job_id
        #return response.json()  # Devuelve la respuesta en formato JSON
    else:
        proceso_completo()
        time.sleep(2)
        access = os.environ.get("ACCESS_TOKEN_HEDRA")
        #if access:
        
        try:
            process_audio_files(FOLDER_PATH_HED, IMAGE_PATH_HED, RUTA_VIDEO_HED, RATIO_HED, PROMPT_HED, SEED_INPUT_HED, SEED_HED, CONTINUE_FRAGMENT_HED, PROCESS_MULTIPLE_HED)
        except Exception as e:
            raise RuntimeError(f"Error process_audio_files")

        print(f"Error en la solicitud")
        return None



def generar_avatar(
    authorization_token,  # Token de autorización que puede cambiar
    avatar_image,         # Imagen base64 del avatar
    aspect_ratio="16:9",  # Relación de aspecto, editable
    prompt="",            # Descripción del prompt, editable
    seed_input=True,      # Si es True, puedes proporcionar un seed; si es False, el seed es aleatorio
    seed=None,            # Semilla proporcionada si seed_input es True, sino se genera aleatoriamente
    voice_url="",         # URL del archivo de voz editable
):
    # Si seed_input es False, generamos una semilla aleatoria
    if not seed_input:
        seed = random.randint(1000000, 9999999)  # Genera un número aleatorio de 7 dígitos

    # Definir la URL de la API
    url = "https://www.hedra.com/api/app/v1/app/avatars/predict-async"

    # Cabeceras HTTP
    headers = {
        "Host": "www.hedra.com",
        "Connection": "keep-alive",
        "Authorization": f"Bearer {authorization_token}",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/131.0.0.0 Safari/537.36",
        "sec-ch-ua": "\"Google Chrome\";v=\"131\", \"Chromium\";v=\"131\", \"Not_A Brand\";v=\"24\"",
        "Content-Type": "application/json",
        "Accept": "*/*",
        "Origin": "https://www.hedra.com",
        "Accept-Language": "es-ES,es;q=0.9",
        "Accept-Encoding": "gzip, deflate",
    }

    # Crear el cuerpo de la solicitud
    data = {
        "text": "",
        "avatar_image": avatar_image,
        "aspect_ratio": aspect_ratio,
        "avatar_image_input": {
            "prompt": prompt,
            "seed": seed
        },
        "audio_source": "audio",  # Usando TTS como la fuente de audio
        "voice_url": voice_url
    }

    # Realizar la solicitud POST
    response = requests.post(url, headers=headers, data=json.dumps(data))

    # Manejo de la respuesta
    if response.status_code == 200:
        try:
            responses = response.json()
            if "job_id" in responses:
                print(f"Solicitud exitosa.")
                return responses['job_id']
            else:
                raise ValueError("Error inesperado: 'job_id' no encontrado en la respuesta.")
        except Exception as e:
            print("Error procesando la respuesta exitosa:", str(e))
            raise

    elif response.status_code == 400:
        try:
            error_detail = response.json().get("detail", {})
            if isinstance(error_detail, dict):
                message = error_detail.get("message", "")
                if "celebrity_detected" in message:
                    print(f"Error de seguridad: {message}")
                    raise RuntimeError("Proceso detenido debido a un problema de seguridad.")
            else:
                print(f"Error inesperado en el detalle del mensaje: {error_detail}")
                raise ValueError("Error desconocido en la clave 'detail'.")

        except Exception as e:
            print("Error procesando la respuesta de error 400:", str(e))
            raise

    elif response.status_code == 402:
        # Si se recibe un código 402 con el mensaje de límite de generaciones alcanzado
        #print("402", response.json().get("detail", {}))
        error_detail = response.json().get("detail", {})
        #if error_detail in "Limit of 5 daily generations reached":
        if True:
            print("Límite de generaciones alcanzado. Ejecutando proceso completo.")
            # Detener el proceso y ejecutar la función proceso_completo
            proceso_completo()
            time.sleep(2)  # Pausa para dar tiempo al proceso
            access = os.environ.get("ACCESS_TOKEN_HEDRA")
            if access:
                try:
                    # Aquí ejecutamos el procesamiento de archivos de audio
                    process_audio_files(
                        FOLDER_PATH_HED, IMAGE_PATH_HED, RUTA_VIDEO_HED, 
                        RATIO_HED, PROMPT_HED, SEED_INPUT_HED, SEED_HED, 
                        CONTINUE_FRAGMENT_HED, PROCESS_MULTIPLE_HED
                    )
                except Exception as e:
                    raise RuntimeError("Error en el proceso process_audio_files") from e
        else:
            print(f"Error inesperado con código 402: {error_detail}")
            raise RuntimeError(f"Fallo en la solicitud HTTP: {response.status_code}")

    else:
        print(f"Error en la solicitud: {response.status_code}, {response.text}")
        raise RuntimeError(f"Fallo en la solicitud HTTP: {response.status_code}")



def upload_audio(file_path, token):
    url = "https://www.hedra.com/api/app/v1/app/avatars/audio"
    headers = {
        "Authorization": f"Bearer {token}",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/127.0.0.0 Safari/537.36",
        "Accept": "*/*",
        "Origin": "https://www.hedra.com",
        "Sec-Fetch-Site": "same-origin",
        "Sec-Fetch-Mode": "cors",
        "Sec-Fetch-Dest": "empty",
        "Referer": "https://www.hedra.com/app/characters",
        "Accept-Language": "es-ES,es;q=0.9",
        "Connection": "keep-alive",
    }

    with open(file_path, 'rb') as audio_file:
        files = {'file': (file_path.split('/')[-1], audio_file, 'audio/wav')}
        response = requests.post(url, headers=headers, files=files)

    if response.status_code == 200:
        print(f"Audio {file_path} uploaded successfully.")
        # Convertimos el texto a un diccionario
        response_dict = json.loads(response.text)

        # Extraemos la URL
        url = response_dict["url"]

        return url
    else:
        print(f"Error uploading audio {file_path}: {response.status_code}, {response.text}")


def process_audio_files(folder_path, image_path, ruta_videos, aspect_ratio, prompt, seed_input, seed, continue_fragment, process_multiple):
    global FOLDER_PATH_HED, IMAGE_PATH_HED, RUTA_VIDEO_HED, RATIO_HED, PROMPT_HED, SEED_INPUT_HED, SEED_HED, CONTINUE_FRAGMENT_HED

    # Asignar valores a las variables globales
    FOLDER_PATH_HED = folder_path
    IMAGE_PATH_HED = image_path
    RUTA_VIDEO_HED = ruta_videos
    RATIO_HED = aspect_ratio
    PROMPT_HED = prompt
    SEED_INPUT_HED = seed_input
    SEED_HED = seed
    CONTINUE_FRAGMENT_HED = continue_fragment

    # Validar el token de acceso
    access_token = os.environ.get("ACCESS_TOKEN_HEDRA")
    if not access_token:
        raise ValueError("El token de acceso (ACCESS_TOKEN_HEDRA) no está configurado en las variables de entorno.")

    # Validar que la carpeta existe.
    if not os.path.exists(folder_path):
        raise FileNotFoundError(f"La carpeta {folder_path} no existe.")

    # Listar archivos de audio
    file_list = sorted([f for f in os.listdir(folder_path) if f.endswith('.mp3')])
    if not file_list:
        print("No se encontraron archivos .mp3 en la carpeta proporcionada. Deteniendo el proceso.")
        return  # Detiene la ejecución si no hay archivos .mp3

    # Si no se debe continuar con fragmentos ni procesar múltiples, procesar solo el primer archivo
    if not continue_fragment and not process_multiple:
        if len(file_list) > 1:
            file_list = file_list[:1]  # Procesar solo el primer archivo

    # Procesamiento de archivos
    for file_name in file_list:
        file_path = os.path.join(folder_path, file_name)
        print(f"Processing audio: {file_path}")

        # Subir audio
        try:
            url_audio = upload_audio(file_path, access_token)
        except Exception as e:
            raise RuntimeError(f"Error al subir el audio {file_path}: {e}")

        file_name_without_extension = os.path.splitext(file_name)[0]

        # Convertir imagen a base64
        try:
            base64_string = image_to_base64(image_path)
        except Exception as e:
            raise RuntimeError(f"Error al convertir la imagen {image_path} a base64: {e}")

        access_token0 = os.environ.get("ACCESS_TOKEN_HEDRA")
        # Generar avatar
        try:
            joob_id = generar_avatar(
                authorization_token=access_token0,
                avatar_image=base64_string,
                aspect_ratio=aspect_ratio,
                prompt=prompt,
                seed_input=seed_input,
                seed=seed,
                voice_url=url_audio
            )
            if not joob_id:
                raise ValueError("El ID del trabajo (joob_id) no se generó correctamente.")
        except Exception as e:
            print("Error al generar el avatar para el audio")
            continue  # Salta al siguiente archivo en lugar de terminar el proceso

        # Obtener progreso
        try:
            resultado = obtener_avatar_sin_cookies(access_token)
            if 'projects' not in resultado or len(resultado['projects']) == 0:
                raise ValueError("No se encontraron proyectos en la respuesta del servidor.")
            else:
                project = resultado['projects'][0]
                initial_progress = project['progress']
                update_progress_bar(access_token, initial_progress, file_name_without_extension, joob_id, ruta_videos, image_path, continue_fragment, process_multiple)

        except Exception as e:
            print(f"Error al obtener el progreso inicial del avatar: {e}")
            continue  # Si hay error en progreso, salta al siguiente archivo

        # Salir si no se continúan fragmentos
        if not continue_fragment and not process_multiple:
            break

        time.sleep(2)

    print("Videos generados con exito...")







import requests
import json
import base64
from PIL import Image
from io import BytesIO
import random
import os

def guardar_imagen_base64(base64_string):
    ruta_archivo = '/tmp/avatar_img.jpg'
    with open(ruta_archivo, 'wb') as archivo_imagen:
        archivo_imagen.write(base64.b64decode(base64_string))
    print(f'Imagen guardada en {ruta_archivo}')

def generar_numero_aleatorio():
    return random.randint(1000000, 9999999)

def enviar_solicitud(prompt="Andrés male, Activista de Derechos Humanos", aspect_ratio="16:9",
                     controlnet_scale=0.7,
                     steps=30,
                     eta=0.8,
                     guidance_scale=0,
                     use_manual_seed=False,
                     seed_number=0,
                     authorization_token="",
                     negative_prompt=False,  # Cambié el valor por defecto a False
                     negative_prompt_txt=""):
    # Decide el valor de la semilla
    seed = seed_number if use_manual_seed else generar_numero_aleatorio()

    # Decide el valor del negative_prompt
    negative_prompt_value = negative_prompt_txt if negative_prompt else ""
    if use_manual_seed:
        os.environ["SEED_HEDRA"] = str(seed)
    else:
        os.environ["SEED_HEDRA"] = "0"

    print(f"Usando seed: {seed}")
    print(f"Usando negative_prompt: {negative_prompt_value}")

    url = "https://www.hedra.com/api/diffusion/v1/models/t2a/predict"
    headers = {
        "Host": "www.hedra.com",
        "Connection": "keep-alive",
        "Enable-Canary": "False",
        "sec-ch-ua-platform": "\"Windows\"",
        "Authorization": f"Bearer {authorization_token}",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/131.0.0.0 Safari/537.36",
        "sec-ch-ua": "\"Google Chrome\";v=\"131\", \"Chromium\";v=\"131\", \"Not_A Brand\";v=\"24\"",
        "Content-Type": "application/json",
        "sec-ch-ua-mobile": "?0",
        "Accept": "*/*",
        "Origin": "https://www.hedra.com",
        "Sec-Fetch-Site": "same-origin",
        "Sec-Fetch-Mode": "cors",
        "Sec-Fetch-Dest": "empty",
        "Referer": "https://www.hedra.com/app/characters",
        "Accept-Language": "es-ES,es;q=0.9",
        "Accept-Encoding": "gzip, deflate"
    }
    data = {
        "data": {
            "prompt": prompt,
            "negative_prompt": negative_prompt_value,
            "mvp_matrix": "1.3025840520858765,-0.005124199669808149,0.031128384172916412,0.031122159212827682,5.651839231275624e-19,1.29294753074646,0.12536752223968506,0.1253424435853958,0.040881600230932236,0.1632690578699112,-0.9918236136436462,-0.9916252493858337,0.000049583341024117544,-0.000011476542567834258,19.803979873657227,20",
            "aspect_ratio": aspect_ratio,
            "controlnet_scale": controlnet_scale,
            "steps": steps,
            "eta": eta,
            "guidance_scale": guidance_scale,
            "seed": seed,
            "model": "sdxl" #sdxl
        }
    }

    response = requests.post(url, headers=headers, data=json.dumps(data))

    if response.status_code == 200:
        print("Request successful.")
        try:
            # Convertir la imagen base64 en un objeto de imagen
            image_base64 = response.text  # Aquí asumimos que la respuesta es una imagen en base64
            image_data = base64.b64decode(image_base64)
            image = Image.open(BytesIO(image_data))

            # Redimensionar la imagen
            new_size = (image.width // 2, image.height // 2)  # Reducir al 50%
            resized_image = image.resize(new_size)

            # Visualizar la imagen redimensionada en Colab
            #display.display(resized_image)
            return image_base64, seed

        except Exception as e:
            print(f"Error processing image: {str(e)}")
            return f"Error processing image: {str(e)}", None
    else:
        print(f"Error: {response.status_code}")
        return f"Error: {response.status_code}", None

def ejecutar_todo(aspect_ratio, prompt_txt, negative_prompt_txt, use_manual_seed, seed_number, negative_prompt):
    access_token = os.environ.get("ACCESS_TOKEN_HEDRA")
    
    # Ejecutar la solicitud
    base64_string, seed_used = enviar_solicitud(
        prompt=prompt_txt,
        aspect_ratio=aspect_ratio,
        controlnet_scale=0.8,
        steps=15,
        eta=0.7,
        guidance_scale=5.0,
        use_manual_seed=use_manual_seed,
        seed_number=seed_number,
        authorization_token=access_token,
        negative_prompt=negative_prompt,
        negative_prompt_txt=negative_prompt_txt
    )
    
    # Guardar la imagen
    guardar_imagen_base64(base64_string)


import os
from pydub import AudioSegment
import gradio as gr
import subprocess
from PIL import Image
import cv2
import sys
import time
from genavaimg import *
from genavatts import *
from hedavatar import *
from registro import *
from hedlogin import *
import shutil
import signal
import sys
import re
import logging


# Configurar el log para registrar errores
logging.basicConfig(filename="app_errors.log", level=logging.ERROR)

# Suprimir errores visibles en la consola (opcional)
sys.stderr = open(os.devnull, 'w')

# Decorador para manejar errores de forma segura
def safe_execution(func):
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except Exception as e:
            logging.error(f"Error en {func.__name__}: {e}")
            return "Ha ocurrido un error. Por favor, revisa tus entradas y vuelve a intentar."
    return wrapper

# Definir el manejador para la interrupción por teclado
def handle_interrupt(signal, frame):
    print("Gradio detenido manualmente. Ruta protegida.")
    sys.exit(0)

# Registrar el manejador para la señal SIGINT
signal.signal(signal.SIGINT, handle_interrupt)

# Función para cargar un video específico
def load_specific_video():
    specific_video_path = "/tmp/videos/0001.mp4"

    if os.path.exists(specific_video_path):
        #print(f"Cargando video específico: {specific_video_path}")
        return [specific_video_path], "Video cargado con éxito."
    else:
        #print(f"Video no encontrado: {specific_video_path}")
        return [], "El video específico no existe."

# Función para escanear videos en la carpeta y devolver una lista de rutas de videos
def scan_videos():
    video_folder = "/tmp/videos"  # Ruta de la carpeta de videos

    # Crear carpeta si no existe
    if not os.path.exists(video_folder):
        os.makedirs(video_folder)

    # Listar solo archivos de video mp4
    videos = [os.path.join(video_folder, f) for f in os.listdir(video_folder) if f.endswith(".mp4")]

    # Ordenar los videos alfanuméricamente
    videos = sorted(videos, key=lambda x: os.path.basename(x).lower())

    # Diagnóstico: Imprimir las rutas encontradas
    print("Videos encontrados (ordenados):")
    print(videos)

    if not videos:
        return [], "No videos encontrados en la carpeta."
    return videos, f"Se encontraron {len(videos)} videos."

# Función para mostrar videos dinámicamente en la interfaz
def handle_video_output_tts():
    videos, message = scan_videos3()

    # Crear una lista de rutas válidas para los componentes `gr.Video`
    video_urls = [video_path for video_path in videos]

    # Verificación en consola
    print("Rutas de videos válidas para mostrar:")
    print(video_urls)

    # Retornar las rutas válidas para renderizarlas en la interfaz
    return video_urls, message
    
# Función para escanear videos en la carpeta y devolver una lista de rutas de videos
def scan_videos3():
    video_folder = "/tmp/videos_tts"  # Ruta de la carpeta de videos

    # Crear carpeta si no existe
    if not os.path.exists(video_folder):
        os.makedirs(video_folder)

    # Listar solo archivos de video mp4
    videos = [os.path.join(video_folder, f) for f in os.listdir(video_folder) if f.endswith(".mp4")]

    # Ordenar los videos alfanuméricamente
    videos = sorted(videos, key=lambda x: os.path.basename(x).lower())

    # Diagnóstico: Imprimir las rutas encontradas
    print("Videos encontrados (ordenados):")
    print(videos)

    if not videos:
        return [], "No videos encontrados en la carpeta."
    return videos, f"Se encontraron {len(videos)} videos."

# Función para mostrar videos dinámicamente en la interfaz
def handle_video_output():
    videos, message = scan_videos()

    # Crear una lista de rutas válidas para los componentes `gr.Video`
    video_urls = [video_path for video_path in videos]

    # Verificación en consola
    print("Rutas de videos válidas para mostrar:")
    print(video_urls)

    # Retornar las rutas válidas para renderizarlas en la interfaz
    return video_urls, message


def run_generate_img_audio_video(prompt, aspect_ratio, seed_input, seed, continue_fragment, process_multiple):
    Aaccess = os.environ.get("ACCESS_TOKEN_HEDRA")

    if Aaccess:
        # Crear la carpeta 'videos' si no existe
        if not os.path.exists('/tmp/videos'):
            os.makedirs('/tmp/videos')

        # Ejemplo de uso
        folder_path = "/tmp/audios/"
        image_path = "/tmp/avatar_img.jpg"
        ruta_videos = "/tmp/videos/"
        #continue_fragment = False
        
        try:
            process_audio_files(folder_path, image_path, ruta_videos, aspect_ratio, prompt, seed_input, seed, continue_fragment, process_multiple)
        except Exception as e:
            raise RuntimeError(f"Error process_audio_files gradio")

    else:
        proceso_completo()
        Aaccess = os.environ.get("ACCESS_TOKEN_HEDRA")
        if Aaccess:
            run_generate_img_audio_video(prompt, aspect_ratio, seed_input, seed, continue_fragment, process_multiple)

    #directory_video = os.environ.get("VIDEO_PATH_HEDRA")


    videos, message = scan_videos()

    # Crear una lista de rutas válidas para los componentes `gr.Video`
    video_urls = [video_path for video_path in videos]

    # Verificación en consola
    print("Rutas de videos válidas para mostrar:")
    print(video_urls)

    # Retornar las rutas válidas para renderizarlas en la interfaz
    return video_urls, message

    # Devuelve la ruta del video y un mensaje
    #return directory_video, "Video creado con éxito"


def join_videos_and_clean(video_folder, output_file):
    try:
        # Verificar si la carpeta existe
        if not os.path.exists(video_folder):
            return "Error: La carpeta no existe.", None

        # Función para dividir texto en partes alfanuméricas y numéricas
        def natural_sort_key(text):
            return [
                int(part) if part.isdigit() else part.lower()
                for part in re.split(r'(\d+)', text)
            ]

        # Obtener y ordenar archivos MP4 con orden natural
        video_files = sorted(
            [f for f in os.listdir(video_folder) if f.endswith(".mp4")],
            key=natural_sort_key
        )

        if not video_files:
            return "Error: No se encontraron archivos MP4 en la carpeta.", None

        # Crear archivo de lista para FFmpeg
        list_file_path = os.path.join(video_folder, "file_list.txt")
        with open(list_file_path, "w") as list_file:
            for video in video_files:
                list_file.write(f"file '{os.path.join(video_folder, video)}'\n")

        # Ejecutar FFmpeg para unir los videos
        command = [
            "ffmpeg",
            "-f", "concat",
            "-safe", "0",
            "-i", list_file_path,
            "-c", "copy",
            output_file
        ]
        subprocess.run(command, check=True)

        # Eliminar archivo de lista temporal
        os.remove(list_file_path)

        # Eliminar los archivos MP4 de la carpeta después de la unión
        for video in video_files:
            os.remove(os.path.join(video_folder, video))

        return "Videos unidos y archivos eliminados exitosamente.", output_file
    except subprocess.CalledProcessError as e:
        return f"Error al unir los videos con FFmpeg: {str(e)}", None
    except Exception as e:
        return f"Error inesperado: {str(e)}", None


# Función conectada al botón Save
def run_save_and_display():

    video_folder = "/tmp/videos"
    output_file = "/tmp/final_video.mp4"  # Archivo final fuera de la carpeta
    message, output_path = join_videos_and_clean(video_folder, output_file)

    if os.path.exists(output_file):
        print(f"Cargando video específico: {output_file}")
        return [output_file], "Video cargado con éxito."
    else:
        print(f"Video no encontrado: {output_file}")
        return [], "El video específico no existe."



# Función para mostrar o ocultar el segundo cuadro de imagen
def activar_end_frame(activar):
    return gr.update(visible=activar)


def run_generate_tts_video(prompt_txt, text, aspect_ratio, selected_tts, use_manual_seed, seed):
    Aaccess = os.environ.get("ACCESS_TOKEN_HEDRA")

    if Aaccess:
        os.makedirs('/tmp/videos_tts', exist_ok=True)

        selected_voice_orignen_id = obtener_id(selected_tts)
        #prompt_txt = ""

        procesar_avatar_request(
            selected_voice_orignen_id,
            use_manual_seed,
            seed,
            aspect_ratio,
            prompt_txt,
            text
        )

    else:
        proceso_completo()
        Aaccess = os.environ.get("ACCESS_TOKEN_HEDRA")
        if Aaccess:
            run_generate_tts_video(prompt_txt, text, aspect_ratio, selected_tts, use_manual_seed, seed)





    #directory_video = os.environ.get("VIDEO_PATH_HEDRA")

    videos, message = scan_videos3()

    # Crear una lista de rutas válidas para los componentes `gr.Video`
    video_urls = [video_path for video_path in videos]

    # Verificación en consola
    print("Rutas de videos válidas para mostrar:")
    print(video_urls)

    # Retornar las rutas válidas para renderizarlas en la interfaz
    return video_urls, message


def run_generate_avatar(prompt_txt, aspect_ratio, negative_prompt_txt, use_manual_seed, seed_number, negative_prompt):

    # Llamar a la función principal
    ejecutar_todo(aspect_ratio, prompt_txt, negative_prompt_txt, use_manual_seed, seed_number, negative_prompt)

    #seed_hedra = os.environ.get("SEED_HEDRA")

    # Si no se usa el seed manual, cargar el seed_hedra
    if not use_manual_seed:
        seed_hedra = os.environ.get("SEED_HEDRA", "default_seed_value")  # Valor por defecto si SEED_HEDRA no está definido
        seed_number = seed_hedra

    # Devuelve la ruta del video y un mensaje
    return '/tmp/avatar_img.jpg', "Avatar creado con éxito"

# Función para procesar y guardar la imagen automáticamente en formato JPG
def process_and_save_image1(image):

    jpg_path = "/tmp/avatar_img.jpg"

    try:
        if image.format != "JPEG":
            image = image.convert("RGB")
        image.save(jpg_path, "JPEG", quality=100)
        if os.path.exists(jpg_path):
            print(f"Imagen guardada correctamente")

            # Obtener las dimensiones de la imagen
            width, height = image.size

            os.environ["WIDTH_IMG1"] = str(width)
            os.environ["HEIGHT_IMG1"] = str(height)
            os.environ["IMG1"] = jpg_path
            print(jpg_path)

            print(f"Coordenadas guardadas correctamente")

        return None
    except Exception as e:
        print(f"Error al procesar la imagen: {str(e)}")
        return None

# Mapeo de nombres visibles a IDs
nombre_a_id = {
    "Commercial Lady (gender: female)": "c2ac25f9-ecc4-4f56-9095-651354df60c0",
    "Laura (gender: female)": "FGY2WhTYpPnrIDTdsKH5",
    "George (gender: male)": "JBFqnCBsd6RMkjVDRZzb",
    "Emily (gender: female)": "LcfcDJNUP1GQjkzn1xUU",
    "Charlotte (gender: female)": "XB0fDUnXU5powFXDhCwa",
    "Alice (gender: female)": "Xb7hH8MSUJpSbSDYk0k2",
    "Matilda (gender: female)": "XrExE9yKIg1WjnnlVkGX",
    "Will (gender: male)": "bIHbv24MWmeRgasZH58o",
    "Brian (gender: male)": "nPczCjzI2devNBz1zQrb",
    "Lily (gender: female)": "pFZP5JQG7iQjIQuC4Bku",
    "Todd - Universal Crossover (gender: male)": "AQ6yxtsTonfHLHY2zUcO",
    "Dr. Von Fusion (gender: male)": "Mg1264PmwVoIedxsF9nu",
    "Kawaii Aerisita (gender: female)": "vGQNBgLaiM3EdZtxIiuY",
    "Lisa Kim (gender: female)": "VjguIG2t2hwOlsU1ftxC",
}

# Función para devolver el ID basado en la elección
def obtener_id(eleccion):
    return nombre_a_id.get(eleccion, "ID no encontrado")



# Crear la carpeta de salida si no existe
output_folder = '/tmp/audios'
os.makedirs(output_folder, exist_ok=True)

def eliminar_contenido_carpetas():
    carpetas = ['/tmp/audios']

    for carpeta in carpetas:
        if os.path.exists(carpeta):
            shutil.rmtree(carpeta)
            os.makedirs(carpeta)

def procesar_audio(file, continue_fragment, process_multiple):
    """Procesa uno o varios archivos MP3 y los divide en fragmentos o los guarda completos según la configuración."""
    eliminar_contenido_carpetas()

    output_folder = "/tmp/audios"  # Carpeta de salida

    if not os.path.exists(output_folder):
        os.makedirs(output_folder)

    try:
        # Si es una lista de archivos
        if process_multiple:
            if not isinstance(file, list):
                return ["Por favor, carga una lista de archivos cuando 'Procesar Múltiples' esté activo."]

            output_files = []
            for audio_file in file:
                if not os.path.exists(audio_file):
                    return [f"Archivo no encontrado: {audio_file}"]

                audio = AudioSegment.from_mp3(audio_file)
                output_filename = os.path.join(output_folder, os.path.basename(audio_file))
                audio.export(output_filename, format='mp3')
                output_files.append(output_filename)

            return ["Archivos procesados y guardados en /tmp/audios:"] + output_files

        # Si es un solo archivo
        else:
            if isinstance(file, list):
                if len(file) != 1:
                    return ["Por favor, carga solo un archivo si 'Procesar Múltiples' no está activo."]
                file = file[0]  # Extraer el único archivo de la lista

            if not os.path.exists(file):
                return [f"Archivo no encontrado: {file}"]

            audio = AudioSegment.from_mp3(file)

            if continue_fragment:
                # Fragmentar el audio en segmentos de 30 segundos
                duration = 30 * 1000  # 30 segundos en milisegundos
                output_files = []

                for i, chunk in enumerate(range(0, len(audio), duration)):
                    segment = audio[chunk:chunk + duration]
                    output_filename = os.path.join(output_folder, f'{i + 1:04d}.mp3')
                    segment.export(output_filename, format='mp3')
                    output_files.append(output_filename)

                return ["Archivos fragmentados y guardados en /tmp/audios:"] + output_files
            else:
                # Guardar el audio completo
                output_filename = os.path.join(output_folder, '0001.mp3')
                audio.export(output_filename, format='mp3')
                return ["Archivo completo guardado en /tmp/audios:", output_filename]

    except Exception as e:
        return [f"Error procesando audio: {e}"]

# Función para crear el archivo .zip de la carpeta de imágenes
def create_zip_of_videos():
        # Crear el archivo zip
        zip_filename = "/tmp/Video_Compressed.zip"
        shutil.make_archive(zip_filename.replace(".zip", ""), 'zip', "/tmp/videos")  # Comprimir la carpeta

        # Devolver el archivo comprimido para ser descargado
        return zip_filename

def on_save_button_zip():
        # Llamar a la función para crear el archivo zip
        zip_file = create_zip_of_videos()

        # Devolver el archivo para que el componente gr.File pueda manejarlo
        return zip_file  # El archivo zip será devuelto al componente gr.File para su descarga


# Función para crear el archivo .zip de la carpeta de imágenes
def create_zip_of_videostts():
        # Crear el archivo zip
        zip_filename = "/tmp/tts_Video_Compressed.zip"
        shutil.make_archive(zip_filename.replace(".zip", ""), 'zip', "/tmp/videos_tts")  # Comprimir la carpeta

        # Devolver el archivo comprimido para ser descargado
        return zip_filename

def on2_save_button_zip():
        # Llamar a la función para crear el archivo zip
        zip_file = create_zip_of_videostts()

        # Devolver el archivo para que el componente gr.File pueda manejarlo
        return zip_file  # El archivo zip será devuelto al componente gr.File para su descarga
# Definir la interfaz
with gr.Blocks() as demo:
    # Título
    gr.HTML(f"<h1 style='text-align: center; font-size: 3em;'>HEDRA 4.0 - AUTOMATIC</p>Created by:<a href='https://www.youtube.com/@IA.Sistema.de.Interes' target='_blank'>IA(Sistema de Interés)</a></p>")
    
    with gr.Tabs():
        # Primera pestaña: Text to Video
        with gr.Tab("Text to Speech - Video Avatar"):
            with gr.Row():
                with gr.Column(scale=1):
                    upload_img = gr.Image(type="pil", label="Upload Image", elem_id="upload_img", interactive=True, visible=False)
                    img0 = gr.Image(type="pil", label="Avatar", elem_id="img0", interactive=False)
                    load_image_checkbox = gr.Checkbox(label="Load Image", value=False, elem_id="load_image_checkbox")
                    aspect_ratio_dropdown = gr.Dropdown(choices=["16:9", "9:16", "1:1"], label="Aspect Ratio", value="16:9", elem_id="aspect_ratio_dropdown")
                    prompt_txt = gr.Textbox(label="Prompt", lines=4, elem_id="prompt_txt")
                    negative_prompt = gr.Checkbox(label="Negative Prompt", value=False, visible=True, elem_id="Negative")
                    negative_prompt_txt = gr.Textbox(label="Negative Prompt", placeholder="Enter text", elem_id="Negative Prompt", visible=False)
                    use_manual_seed = gr.Checkbox(label="Add Seed", value=False, visible=True, elem_id="Add_Seed")
                    seed_number = gr.Textbox(label="Seed", placeholder="Enter text", elem_id="Seed", visible=False)
                    generate_avatar_button = gr.Button("Generate Avatar", elem_id="generate_avatar_button")
                    aspect_tts = gr.Dropdown(choices=["Laura (gender: female)", "John (gender: male)"], label="Text to Speech TTS", value="Laura (gender: female)", elem_id="aspect_tts")
                    audio_text = gr.Textbox(label="Audio TTS write here (maximum 300 characters)", max_length=300, lines=4, elem_id="audio_text")
                    create_button3 = gr.Button("Generate Video Avatar", elem_id="create_button3")
                    save_button0 = gr.Button("Compress all videos", elem_id="save_button3")
                    load_button = gr.Button("Load video list", elem_id="create_button3")
                
                with gr.Column(scale=2):
                    video_outputs3 = gr.Gallery([])
                    output_textbox3 = gr.Textbox(label="Output", interactive=False, elem_id="output_textbox3")
                    download_log = gr.File(label="Download Zip", elem_id="download_log")

            load_image_checkbox.change(
                lambda x: [gr.update(visible=x), gr.update(visible=not x), gr.update(visible=not x), gr.update(visible=not x), gr.update(visible=(not x and negative_prompt.value)), gr.update(visible=not x)], 
                load_image_checkbox, 
                [upload_img, img0, prompt_txt, negative_prompt, negative_prompt_txt, generate_avatar_button]
            )
            negative_prompt.change(lambda x: gr.update(visible=x), negative_prompt, negative_prompt_txt)
            use_manual_seed.change(activar_end_frame, use_manual_seed, seed_number)

            create_button3.click(
                fn=safe_execution(run_generate_tts_video),
                inputs=[prompt_txt, audio_text, aspect_ratio_dropdown, aspect_tts, use_manual_seed, seed_number],
                outputs=[video_outputs3, output_textbox3]
            )

            upload_img.change(fn=safe_execution(process_and_save_image1), inputs=[upload_img], outputs=[])

            generate_avatar_button.click(
                fn=safe_execution(run_generate_avatar),
                inputs=[prompt_txt, aspect_ratio_dropdown, negative_prompt_txt, use_manual_seed, seed_number, negative_prompt],
                outputs=[img0, output_textbox3]
            )

            load_button.click(fn=safe_execution(handle_video_output_tts), inputs=[], outputs=[video_outputs3, output_textbox3])
            save_button0.click(fn=safe_execution(on2_save_button_zip), outputs=download_log)

        # Segunda pestaña: Image+Audio to Video Avatar
        with gr.Tab("Image+Audio to Video Avatar"):
            with gr.Row():
                with gr.Column(scale=1):
                    img1 = gr.Image(type="pil", label="Drag image here or select image", interactive=True, elem_id="img1")
                    img2 = gr.Image(type="pil", label="Upload the last frame image", interactive=True, elem_id="img2", visible=False, width=200, height=200)
                    inputs_audio = gr.File(label="Sube tu archivo MP3 o lista de MP3", type="filepath", file_types=[".mp3"], file_count="multiple")
                    aspect_ratio_dropdown = gr.Dropdown(choices=["16:9", "9:16", "1:1"], label="Aspect Ratio", value="16:9", elem_id="aspect_ratio_dropdown")
                    subject = gr.Textbox(label="Prompt", placeholder="Prompt text", elem_id="description_input")
                    seed_input = gr.Checkbox(label="Seed Manual", value=False, visible=True)
                    Seed_text = gr.Textbox(label="Seed", placeholder="Enter seed", elem_id="seed_input", visible=False)
                    continue_fragment = gr.Checkbox(label="Continue Fragment", value=False, visible=False, elem_id="loop_video")
                    process_multiple = gr.Checkbox(label="Procesar múltiples archivos sin dividir", value=False)
                    prompt_upsampling = gr.Checkbox(label="prompt upsampling", value=True, visible=False, elem_id="prompt_upsampling")
                    create_button = gr.Button("Generate Video Avatar", elem_id="create_button")
                    save_button = gr.Button("Render", elem_id="save_button")
                    load_button = gr.Button("Load video list", elem_id="create_button3")
                    save_button3 = gr.Button("Compress all videos", elem_id="compress_button")
                
                with gr.Column(scale=2):
                    video_outputs = gr.Gallery([])
                    output_textbox = gr.Textbox(label="Output", interactive=False, elem_id="output_textbox")
                    download_log = gr.File(label="Download Zip", elem_id="download_log")

            inputs_audio.change(fn=safe_execution(procesar_audio), inputs=[inputs_audio, continue_fragment, process_multiple], outputs=[output_textbox])
            seed_input.change(activar_end_frame, seed_input, Seed_text)
            img1.change(fn=safe_execution(process_and_save_image1), inputs=[img1], outputs=[])
            load_button.click(fn=safe_execution(handle_video_output), inputs=[], outputs=[video_outputs, output_textbox])
            create_button.click(fn=safe_execution(run_generate_img_audio_video), inputs=[subject, aspect_ratio_dropdown, seed_input, Seed_text, continue_fragment, process_multiple], outputs=[video_outputs, output_textbox])
            save_button.click(fn=safe_execution(run_save_and_display), inputs=[], outputs=[video_outputs, output_textbox])
            save_button3.click(fn=safe_execution(on_save_button_zip), outputs=download_log)

# Ejecutar la interfaz
demo.launch(inline=False, debug=False, share=True)


import requests
import urllib3
from urllib.parse import urlencode
import re
import time
import json
import os

def get_session_info():
    url = "https://www.hedra.com/api/auth/csrf"
    headers = {
        "Host": "www.hedra.com",
        "Connection": "keep-alive",
        "sec-ch-ua": '"Not)A;Brand";v="99", "Google Chrome";v="127", "Chromium";v="127"',
        "Content-Type": "application/json",
        "sec-ch-ua-mobile": "?0",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/127.0.0.0 Safari/537.36",
        "sec-ch-ua-platform": '"Windows"',
        "Accept": "*/*",
        "Sec-Fetch-Site": "same-origin",
        "Sec-Fetch-Mode": "cors",
        "Sec-Fetch-Dest": "empty",
        "Referer": "https://www.hedra.com/login?redirectUrl=%2F&ref=nav",
        "Accept-Language": "es-ES,es;q=0.9",
        "Accept-Encoding": "gzip, deflate"
    }

    try:
        response = requests.get(url, headers=headers)
        response.raise_for_status()
        cookies = response.cookies
        formatted_cookies = "; ".join([f"{cookie.name}={cookie.value}" for cookie in cookies])
        csrf_token = response.json().get('csrfToken')
        return csrf_token, formatted_cookies
    except requests.exceptions.RequestException:
        return None, None


def get_session_info2(formatted_cookies):
    url = "https://www.hedra.com/api/auth/session"
    headers = {
        "Host": "www.hedra.com",
        "Connection": "keep-alive",
        "sec-ch-ua": '"Not)A;Brand";v="99", "Google Chrome";v="127", "Chromium";v="127"',
        "Content-Type": "application/json",
        "sec-ch-ua-mobile": "?0",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/127.0.0.0 Safari/537.36",
        "sec-ch-ua-platform": '"Windows"',
        "Accept": "*/*",
        "Sec-Fetch-Site": "same-origin",
        "Sec-Fetch-Mode": "cors",
        "Sec-Fetch-Dest": "empty",
        "Referer": "https://www.hedra.com/login?redirectUrl=%2F&ref=nav",
        "Accept-Language": "es-ES,es;q=0.9",
        "Cookie": f"{formatted_cookies}",
        "Accept-Encoding": "gzip, deflate"
    }

    try:
        response = requests.get(url, headers=headers)
        response.raise_for_status()
        cookies = response.cookies
        csrf_token = None
        for cookie in cookies:
            if cookie.name == "__Secure-next-auth.session-token":
                csrf_token = cookie.value
                break
        return csrf_token
    except requests.exceptions.RequestException:
        return None


def post_sign_in(txtCorreo, txtContra, formatted_cookies, session_token, csrf_token):
    url = "https://www.hedra.com/api/auth/callback/credentials"
    headers = {
        "Host": "www.hedra.com",
        "Connection": "keep-alive",
        "sec-ch-ua": '"Not)A;Brand";v="99", "Google Chrome";v="127", "Chromium";v="127"',
        "Content-Type": "application/x-www-form-urlencoded",
        "sec-ch-ua-mobile": "?0",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/127.0.0.0 Safari/537.36",
        "sec-ch-ua-platform": '"Windows"',
        "Accept": "*/*",
        "Origin": "https://www.hedra.com",
        "Sec-Fetch-Site": "same-origin",
        "Sec-Fetch-Mode": "cors",
        "Sec-Fetch-Dest": "empty",
        "Referer": "https://www.hedra.com/login?redirectUrl=%2F&ref=nav",
        "Accept-Language": "es-ES,es;q=0.9",
        "Cookie": f"{formatted_cookies}; __Secure-next-auth.session-token={session_token}",
        "Accept-Encoding": "gzip, deflate"
    }

    data = {
        "email": txtCorreo,
        "password": txtContra,
        "action": "signIn",
        "redirect": "false",
        "csrfToken": csrf_token,
        "callbackUrl": "https://www.hedra.com/login?redirectUrl=%2F&ref=nav",
        "json": "true"
    }

    encoded_data = urlencode(data)

    http = urllib3.PoolManager()
    try:
        response = http.request('POST', url, body=encoded_data, headers=headers)
        if response.status == 200 and 'set-cookie' in response.headers:
            cookies = response.headers['set-cookie']
            match_0 = re.search(r'__Secure-next-auth.session-token.0=([^;]+)', cookies)
            match_1 = re.search(r'__Secure-next-auth.session-token.1=([^;]+)', cookies)
            session_token_0 = match_0.group(1) if match_0 else None
            session_token_1 = match_1.group(1) if match_1 else None
            return session_token_0, session_token_1
    except Exception:
        pass
    return None, None


def obtener_session(url, token_0, token_1, csrf_Cookies):
    headers = {
        'Host': 'www.hedra.com',
        'Connection': 'keep-alive',
        'sec-ch-ua-platform': '"Windows"',
        'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/127.0.0.0 Safari/537.36',
        'sec-ch-ua': '"Google Chrome";v="127", "Chromium";v="127", "Not_A Brand";v="99"',
        'Content-Type': 'application/json',
        'sec-ch-ua-mobile': '?0',
        'Accept': '*/*',
        'Sec-Fetch-Site': 'same-origin',
        'Sec-Fetch-Mode': 'cors',
        'Sec-Fetch-Dest': 'empty',
        'Referer': 'https://www.hedra.com/login?redirectUrl=%2Fapp%2Fcharacters&ref=nav',
        'Accept-Language': 'es-ES,es;q=0.9',
        'Cookie': f'__Host-next-auth.csrf-token={csrf_Cookies}; __Secure-next-auth.session-token.0={token_0}; __Secure-next-auth.session-token.1={token_1}',
        'Accept-Encoding': 'gzip, deflate',
    }

    try:
        response = requests.get(url, headers=headers)
        if response.status_code == 200:
            data = response.json()
            return data.get('user', {}).get('accessToken', None)
    except Exception:
        pass
    return None


def ejecutar_proceso_completo(email, password):
    try:
        csrf_token, formatted_cookies = get_session_info()
        session_token = get_session_info2(formatted_cookies)
        token_0, token_1 = post_sign_in(email, password, formatted_cookies, session_token, csrf_token)
        api_url = 'https://www.hedra.com/api/auth/session'
        access_token = obtener_session(api_url, token_0, token_1, formatted_cookies)
        if access_token:
            os.environ["ACCESS_TOKEN_HEDRA"] = access_token

        return "access_token"
    except Exception:
        return None


import requests
import re
from bs4 import BeautifulSoup
import time
import urllib3
from urllib.parse import urlencode
import re
import json
import ipywidgets as widgets
import random
import string
from datetime import datetime
import os
import random
import string

def generar_contrasena():
    # Definir los conjuntos de caracteres
    minusculas = string.ascii_lowercase
    mayusculas = string.ascii_uppercase
    numeros = string.digits
    caracteres_especiales = string.punctuation

    # Asegurarse de que la contraseña contiene al menos uno de cada tipo
    contrasena = [
        random.choice(minusculas),
        random.choice(mayusculas),
        random.choice(numeros),
        random.choice(caracteres_especiales)
    ]

    # Completar la contraseña hasta tener al menos 8 caracteres
    todos_caracteres = minusculas + mayusculas + numeros + caracteres_especiales
    contrasena += random.choices(todos_caracteres, k=8 - len(contrasena))

    # Mezclar los caracteres para que el orden sea aleatorio
    random.shuffle(contrasena)

    # Convertir la lista en una cadena
    return ''.join(contrasena)

    
def obtener_fecha_actual():
    return datetime.now().strftime("%b-%d-%Y").lower()  # Formato 'mmm-dd-yyyy', ejemplo: 'nov-20-2024'

def registrar_usuario(token):
    # Obtener la fecha actual para tos_version
    tos_version = obtener_fecha_actual()

    # Variables constantes (no editables)
    tos_accepted = True
    residence_status = "ALLOW"
    marketing_email_consent = "ALLOW"

    api_url = "https://api.dev.dream-ai.com/register"

    # Headers de la solicitud
    headers = {
        'Host': 'api.dev.dream-ai.com',
        'Connection': 'keep-alive',
        'sec-ch-ua-platform': '"Windows"',
        'Authorization': f'Bearer {token}',
        'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/131.0.0.0 Safari/537.36',
        'sec-ch-ua': '"Google Chrome";v="131", "Chromium";v="131", "Not_A Brand";v="24"',
        'Content-Type': 'application/json',
        'sec-ch-ua-mobile': '?0',
        'Accept': '*/*',
        'Origin': 'https://www.hedra.com',
        'Sec-Fetch-Site': 'cross-site',
        'Sec-Fetch-Mode': 'cors',
        'Sec-Fetch-Dest': 'empty',
        'Referer': 'https://www.hedra.com/',
        'Accept-Language': 'es-ES,es;q=0.9',
        'Accept-Encoding': 'gzip, deflate'
    }

    # Cuerpo de la solicitud
    payload = {
        "tos_version": tos_version,
        "tos_accepted": tos_accepted,
        "residence_not_blocked": residence_status,
        "marketing_email_consent": marketing_email_consent
    }

    try:
        # Realizar la solicitud POST
        response = requests.post(api_url, headers=headers, json=payload)

        # Verificar el estado de la respuesta
        if response.status_code == 200:
            return "Respuesta exitosa"
        else:
            return {"error": f"Error en la solicitud: {response.status_code}", "detalle": response.text}
    except Exception as e:
        return {"error": f"Error al realizar la solicitud: {str(e)}"}


def generar_nombre_completo():
  """Genera un nombre completo con un número aleatorio de 3 dígitos."""

  nombres = ["Juan", "Pedro", "Maria", "Ana", "Luis", "Sofia", "Diego", "Laura", "Javier", "Isabel",
            "Pablo", "Marta", "David", "Elena", "Sergio", "Irene", "Daniel", "Alicia", "Carlos", "Sandra",
            "Antonio", "Lucia", "Miguel", "Sara", "Jose", "Cristina", "Alberto", "Blanca", "Alejandro", "Marta",
            "Francisco", "Esther", "Roberto", "Silvia", "Manuel", "Patricia", "Marcos", "Victoria", "Fernando", "Rosa"]
  apellidos = ["Garcia", "Rodriguez", "Gonzalez", "Fernandez", "Lopez", "Martinez", "Sanchez", "Perez", "Alonso", "Diaz",
            "Martin", "Ruiz", "Hernandez", "Jimenez", "Torres", "Moreno", "Gomez", "Romero", "Alvarez", "Vazquez",
            "Gil", "Lopez", "Ramirez", "Santos", "Castro", "Suarez", "Munoz", "Gomez", "Gonzalez", "Navarro",
            "Dominguez", "Lopez", "Rodriguez", "Sanchez", "Perez", "Garcia", "Gonzalez", "Martinez", "Fernandez", "Lopez"]

  nombre = random.choice(nombres)
  apellido = random.choice(apellidos)
  numero = random.randint(100, 999)

  nombre_completo = f"{nombre}_{apellido}_{numero}"
  return nombre_completo

def enviar_formulario():
    """Envía una solicitud POST a un formulario web."""
    url = 'https://email-fake.com/'
    datos = {'campo_correo': 'ejemplo@dominio.com'}
    response = requests.post(url, data=datos)
    return response

def extraer_dominios(response_text):
    """Extrae dominios de un texto utilizando expresiones regulares."""
    dominios = re.findall(r'id="([^"]+\.[^"]+)"', response_text)
    return dominios

def obtener_sitio_web_aleatorio(response_text):
    """Obtiene un sitio web aleatorio de los dominios extraídos."""
    dominios = extraer_dominios(response_text)
    sitio_web_aleatorio = random.choice(dominios)
    return sitio_web_aleatorio

"""def extract_verification_code(html_content):
    soup = BeautifulSoup(html_content, 'html.parser')

    # Buscar el texto que contiene el código de verificación
    code_element = soup.find('div', class_='fem mess_bodiyy').find('p')

    if code_element:
        # Extraer y devolver solo el número
        verification_code = code_element.get_text().strip()
        return verification_code
    else:
        return None"""

def extract_verification_code(html_content):
    try:
        # Crear el objeto BeautifulSoup para analizar el contenido HTML
        soup = BeautifulSoup(html_content, 'html.parser')

        # Intentar buscar el div y el párrafo que contiene el código
        code_element = soup.find('div', class_='fem mess_bodiyy')
        
        # Verificar que el div y el párrafo existen antes de intentar acceder a ellos
        if code_element:
            p_element = code_element.find('p')
            if p_element:
                # Extraer y devolver el texto del párrafo
                verification_code = p_element.get_text().strip()
                return verification_code
        
        # Si no se encontró el código, devolver un mensaje claro
        return "No Exit"

    except Exception as e:
        # Manejar errores inesperados y devolver un mensaje
        return f"Error procesando el HTML: {e}"

def post_register(token):
    url = "https://api.dev.dream-ai.com/register"
    headers = {
        "Host": "api.dev.dream-ai.com",
        "Connection": "keep-alive",
        "sec-ch-ua": '"Not)A;Brand";v="99", "Google Chrome";v="127", "Chromium";v="127"',
        "sec-ch-ua-platform": '"Windows"',
        "sec-ch-ua-mobile": "?0",
        "Authorization": f"Bearer {token}",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/127.0.0.0 Safari/537.36",
        "Content-Type": "application/json",
        "Accept": "*/*",
        "Origin": "https://www.hedra.com",
        "Sec-Fetch-Site": "cross-site",
        "Sec-Fetch-Mode": "cors",
        "Sec-Fetch-Dest": "empty",
        "Referer": "https://www.hedra.com/",
        "Accept-Language": "es-ES,es;q=0.9",
        "Accept-Encoding": "gzip, deflate"
    }
    payload = {
        "tos_version": "may-21-2023",
        "tos_accepted": True,
        "residence_not_blocked": "ALLOW",
        "marketing_email_consent": "NONE"
    }

    # Initialize the HTTP client
    http = urllib3.PoolManager()

    # Send the POST request
    response = http.request(
        'POST',
        url,
        body=json.dumps(payload),
        headers=headers
    )

    # Decode the response
    response_data = json.loads(response.data.decode('utf-8'))
    #print(response_data)

    if response.status == 200:
        print("Registro exitoso!")
        return response_data
    else:
        print(f"Error en el registro: {response.status}")
        return None



def get_session( formatted_cookies, token_0, token_1):
    # Crear un administrador de conexiones
    http = urllib3.PoolManager()

    api_url = 'https://www.hedra.com/api/auth/session'

    # Realizar la solicitud GET sin enviar cookies
    response = http.request(
        'GET',
        api_url,
        headers={
            'Host': 'www.hedra.com',
            'Connection': 'keep-alive',
            'sec-ch-ua': '"Not)A;Brand";v="99", "Google Chrome";v="127", "Chromium";v="127"',
            'Content-Type': 'application/json',
            'sec-ch-ua-mobile': '?0',
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/127.0.0.0 Safari/537.36',
            'sec-ch-ua-platform': '"Windows"',
            'Accept': '*/*',
            'Sec-Fetch-Site': 'same-origin',
            'Sec-Fetch-Mode': 'cors',
            'Sec-Fetch-Dest': 'empty',
            'Referer': 'https://www.hedra.com/login?redirectUrl=%2F&ref=nav',
            'Accept-Language': 'es-ES,es;q=0.9',
            "Cookie": f"{formatted_cookies}; __Secure-next-auth.session-token.0={token_0}; __Secure-next-auth.session-token.1={token_1}",
            "Accept-Encoding": "gzip, deflate"
        }
    )

    data = json.loads(response.data.decode('utf-8'))
    #print(data)

    # Extraer el access_token
    access_token = data.get('user', {}).get('accessToken', None)

    return response.status, access_token



def post_sign_in(txtEmail, txtPass, formatted_cookies, session_token, csrf_token):
    url = "https://www.hedra.com/api/auth/callback/credentials"
    headers = {
        "Host": "www.hedra.com",
        "Connection": "keep-alive",
        "sec-ch-ua": '"Not)A;Brand";v="99", "Google Chrome";v="127", "Chromium";v="127"',
        "Content-Type": "application/x-www-form-urlencoded",
        "sec-ch-ua-mobile": "?0",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/127.0.0.0 Safari/537.36",
        "sec-ch-ua-platform": '"Windows"',
        "Accept": "*/*",
        "Origin": "https://www.hedra.com",
        "Sec-Fetch-Site": "same-origin",
        "Sec-Fetch-Mode": "cors",
        "Sec-Fetch-Dest": "empty",
        "Referer": "https://www.hedra.com/login?redirectUrl=%2F&ref=nav",
        "Accept-Language": "es-ES,es;q=0.9",
        "Cookie": f"{formatted_cookies}; ph_phc_LPkfNqgrjYQMX7vjw63IAdpzDFpLNUz4fSq3dgbMRgS_posthog=%7B%22distinct_id%22%3A%22d8712390-0001-7079-897e-1eb6d2aa371d%22%2C%22%24sesid%22%3A%5B1723321702013%2C%2201913dfa-5168-7dea-8e61-d31f9f65d4ca%22%2C1723321700712%5D%2C%22%24epp%22%3Atrue%7D; __Secure-next-auth.session-token={session_token}",
        "Accept-Encoding": "gzip, deflate"
    }

    data = {
        "email": f"{txtEmail}",
        "password": f"{txtPass}",
        "action": "signIn",
        "redirect": "false",
        "csrfToken": f"{csrf_token}",
        "callbackUrl": "https://www.hedra.com/login?redirectUrl=%2F&ref=nav",
        "json": "true"
    }

    encoded_data = urlencode(data)

    http = urllib3.PoolManager()
    try:
        response = http.request(
            'POST',
            url,
            body=encoded_data,
            headers=headers
        )
    except Exception as e:
        print(f"Error during request: {e}")
        return None, None

    if response.status == 200:
        if 'set-cookie' in response.headers:
            cookies = response.headers['set-cookie']
            #print("Cookies recibidas:")
            #print(cookies)

            # Extracción de los valores de las cookies específicas
            session_token_0 = None
            session_token_1 = None

            # Buscar los tokens específicos en las cookies
            match_0 = re.search(r'__Secure-next-auth.session-token.0=([^;]+)', cookies)
            match_1 = re.search(r'__Secure-next-auth.session-token.1=([^;]+)', cookies)

            if match_0:
                session_token_0 = match_0.group(1)
            if match_1:
                session_token_1 = match_1.group(1)

            # Imprimir los valores extraídos
            #if session_token_0:
             #   print(f"__Secure-next-auth.session-token.0: {session_token_0}")
            #if session_token_1:
            #    print(f"__Secure-next-auth.session-token.1: {session_token_1}")

            # Retornar los tokens extraídos
            return session_token_0, session_token_1
        else:
            print("No se encontraron cookies en la respuesta.")
            return None, None
    else:
        print(f"Failed to post credentials. Status code: {response.status}")
        return None, None



def get_session_info2(txtEmail, txtPass, formatted_cookies, session_token):
    url = "https://www.hedra.com/api/auth/session"
    headers = {
        "Host": "www.hedra.com",
        "Connection": "keep-alive",
        "sec-ch-ua": '"Not)A;Brand";v="99", "Google Chrome";v="127", "Chromium";v="127"',
        "Content-Type": "application/json",
        "sec-ch-ua-mobile": "?0",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/127.0.0.0 Safari/537.36",
        "sec-ch-ua-platform": '"Windows"',
        "Accept": "*/*",
        "Sec-Fetch-Site": "same-origin",
        "Sec-Fetch-Mode": "cors",
        "Sec-Fetch-Dest": "empty",
        "Referer": "https://www.hedra.com/login?redirectUrl=%2F&ref=nav",
        "Accept-Language": "es-ES,es;q=0.9",
        "Cookie": f"{formatted_cookies}; ph_phc_LPkfNqgrjYQMX7vjw63IAdpzDFpLNUz4fSq3dgbMRgS_posthog=%7B%22distinct_id%22%3A%22d8712390-0001-7079-897e-1eb6d2aa371d%22%2C%22%24sesid%22%3A%5B1723321702013%2C%2201913dfa-5168-7dea-8e61-d31f9f65d4ca%22%2C1723321700712%5D%2C%22%24epp%22%3Atrue%7D; __Secure-next-auth.session-token={session_token}",
        "Accept-Encoding": "gzip, deflate"
    }

    try:
        response = requests.get(url, headers=headers)
        response.raise_for_status()  # Lanza una excepción para códigos de error HTTP
        print("Response received:")
        #print(response.text)

        # Extraer y formatear las cookies
        cookies = response.cookies
        print("Extraer y formatear las cookies...")

        # Buscar la cookie `__Host-next-auth.csrf-token`
        csrf_token = None
        for cookie in cookies:
            if cookie.name == "__Secure-next-auth.session-token":
                csrf_token = cookie.value
                break

        """if csrf_token:
            print(f"__Secure-next-auth.session-token: correct")
        else:
            print("__Secure-next-auth.session-token not found")"""

        return csrf_token

    except requests.exceptions.RequestException as e:
        print(f"Failed to get session info. Error: {e}")
        return None





def post_credentials_with_code(txtEmail, txtPass, codigoverificado, formatted_cookies, session_token, csrf_token):
    url = "https://www.hedra.com/api/auth/callback/credentials"
    headers = {
        "Host": "www.hedra.com",
        "Connection": "keep-alive",
        "sec-ch-ua": '"Not)A;Brand";v="99", "Google Chrome";v="127", "Chromium";v="127"',
        "Content-Type": "application/x-www-form-urlencoded",
        "sec-ch-ua-mobile": "?0",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/127.0.0.0 Safari/537.36",
        "sec-ch-ua-platform": '"Windows"',
        "Accept": "*/*",
        "Origin": "https://www.hedra.com",
        "Sec-Fetch-Site": "same-origin",
        "Sec-Fetch-Mode": "cors",
        "Sec-Fetch-Dest": "empty",
        "Referer": "https://www.hedra.com/login?redirectUrl=%2F&ref=nav",
        "Accept-Language": "es-ES,es;q=0.9",
        "Cookie": f"{formatted_cookies}; ph_phc_LPkfNqgrjYQMX7vjw63IAdpzDFpLNUz4fSq3dgbMRgS_posthog=%7B%22distinct_id%22%3A%22d8712390-0001-7079-897e-1eb6d2aa371d%22%2C%22%24sesid%22%3A%5B1723321702013%2C%2201913dfa-5168-7dea-8e61-d31f9f65d4ca%22%2C1723321700712%5D%2C%22%24epp%22%3Atrue%7D; __Secure-next-auth.session-token={session_token}",
        "Accept-Encoding": "gzip, deflate"
    }

    data = {
        "email": txtEmail,
        "password": txtPass,
        "action": "confirm",
        "code": codigoverificado,
        "redirect": "false",
        "csrfToken": csrf_token,
        "callbackUrl": "https://www.hedra.com/login?redirectUrl=%2F&ref=nav",
        "json": "true"
    }

    try:
        response = requests.post(url, headers=headers, data=data)

        if response.status_code == 200:
            print("Response received:")
            if 'set-cookie' in response.headers:
                cookies = response.headers['set-cookie']
                print("Cookies received:")
                return cookies
        else:
            print(f"Failed to post credentials. Status code: {response.status_code}")
            return None
    except requests.RequestException as e:
        print(f"An error occurred: {e}")
        return None


def post_credentials(txtEmail, txtPass, formatted_cookies, csrf_token):
    url = "https://www.hedra.com/api/auth/callback/credentials"
    headers = {
        "Connection": "keep-alive",
        "sec-ch-ua": '"Not)A;Brand";v="99", "Google Chrome";v="127", "Chromium";v="127"',
        "Content-Type": "application/x-www-form-urlencoded",
        "sec-ch-ua-mobile": "?0",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/127.0.0.0 Safari/537.36",
        "sec-ch-ua-platform": '"Windows"',
        "Accept": "*/*",
        "Origin": "https://www.hedra.com",
        "Sec-Fetch-Site": "same-origin",
        "Sec-Fetch-Mode": "cors",
        "Sec-Fetch-Dest": "empty",
        "Referer": "https://www.hedra.com/login?redirectUrl=%2F&ref=nav",
        "Accept-Language": "es-ES,es;q=0.9",
        "Cookie": formatted_cookies,
        "Accept-Encoding": "gzip, deflate"
    }

    data = {
        "email": txtEmail,
        "password": txtPass,
        "action": "signUp",
        "redirect": "true",
        "csrfToken": csrf_token,
        "callbackUrl": "https://www.hedra.com/login?redirectUrl=%2F&ref=nav",
        "json": "true"
    }

    try:
        response = requests.post(url, headers=headers, data=data)

        if response.status_code == 200:
            print("Response received:")

            if 'set-cookie' in response.headers:
                cookies = response.headers['set-cookie']
                print("Cookies received:")

                match = re.search(r'__Secure-next-auth\.session-token=([^;]+)', cookies)
                if match:
                    session_token = match.group(1)
                    return session_token
                else:
                    print("Session Token not found.")
                    return None
        else:
            print("Failed to post credentials. Retrying...")
            configurar_credenciales()

            print("Obteniendo información de la sesión2...")
            csrf_token1, formatted_cookies1 = get_session_info()

            os.environ["CSRF_TOKEN"] = csrf_token1
            os.environ["FORMATTED_COOKIE"] = formatted_cookies1

            print("Enviando credenciales2...")
            correo = os.environ.get("EMAIL_HEDRA")
            contrasena = os.environ.get("PASS_HEDRA")
            session_token = post_credentials(correo, contrasena, formatted_cookies1, csrf_token1)
            if session_token:
                os.environ["SESSION_TOKEN"] = session_token
                print("Credenciales aceptadas2.")
            return session_token
    except requests.RequestException as e:
        print(f"An error occurred: {e}")
        return None



def get_session_info():
    url = "https://www.hedra.com/api/auth/csrf"
    headers = {
        "Host": "www.hedra.com",
        "Connection": "keep-alive",
        "sec-ch-ua": '"Not)A;Brand";v="99", "Google Chrome";v="127", "Chromium";v="127"',
        "Content-Type": "application/json",
        "sec-ch-ua-mobile": "?0",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/127.0.0.0 Safari/537.36",
        "sec-ch-ua-platform": '"Windows"',
        "Accept": "*/*",
        "Sec-Fetch-Site": "same-origin",
        "Sec-Fetch-Mode": "cors",
        "Sec-Fetch-Dest": "empty",
        "Referer": "https://www.hedra.com/login?redirectUrl=%2F&ref=nav",
        "Accept-Language": "es-ES,es;q=0.9",
        "Accept-Encoding": "gzip, deflate"
    }

    try:
        response = requests.get(url, headers=headers)
        response.raise_for_status()  # Lanza una excepción para códigos de error HTTP
        print("Response received:")
        #print(response.json())  # Cambia a response.text si no es JSON

        # Extraer y formatear las cookies
        cookies = response.cookies
        formatted_cookies = "; ".join([f"{cookie.name}={cookie.value}" for cookie in cookies])

        print("Formatted Cookies:")
        #print(formatted_cookies)

        # Retornar el csrfToken y las cookies formateadas si deseas utilizarlas después
        csrf_token = response.json().get('csrfToken')
        return csrf_token, formatted_cookies

    except requests.exceptions.RequestException as e:
        print(f"Failed to get session info. Error: {e}")



def extract_confirmation_code(text):
    # Utilizar una expresión regular para buscar el número en el texto
    match = re.search(r'\b\d{6}\b', text)
    if match:
        return match.group(0)  # Devolver el número encontrado
    else:
        return None


def enviar_dell_post(id_dell, usuarios, dominios):
    url = 'https://email-fake.com/del_mail.php'#{dominios}%2F{usuario}
    headers = {
       'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8',
       'X-Requested-With': 'XMLHttpRequest',
       'Cookie': f'embx=%5B%22{usuarios}%40{dominios}; surl={dominios}/{usuarios}/',
       'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36',
       'Accept': '*/*',
       'Origin': 'https://email-fake.com',
       'Sec-Fetch-Site': 'same-origin',
       'Sec-Fetch-Mode': 'cors',
       'Sec-Fetch-Dest': 'empty',
       'Accept-Language': 'es-ES,es;q=0.9'
    }

    data = {
       'delll': f'{id_dell}'
    }

    try:
        response = requests.post(url, headers=headers, data=data)
        response.raise_for_status()  # Raise an exception for 4xx/5xx status codes
        return response.text
    except requests.exceptions.RequestException as e:
        return f"Error en la solicitud POST: {str(e)}"

def extract_codes_from_html(html):
    soup = BeautifulSoup(html, 'html.parser')

    # Encuentra la celda <td> con el estilo y clase específicos
    td_tag = soup.find('td', {'class': 'inner-td', 'style': 'border-radius: 6px; font-size: 16px; text-align: center; background-color: inherit'})

    if td_tag:
        # Encuentra la etiqueta <a> dentro de la celda <td>
        a_tag = td_tag.find('a', href=True)

        if a_tag:
            # Obtén el valor del atributo href
            href = a_tag['href']

            # Encuentra el valor de internalCode y oobCode en el href
            internal_code = None
            oob_code = None

            if 'internalCode=' in href:
                internal_code = href.split('internalCode=')[1].split('&')[0]

            if 'oobCode=' in href:
                oob_code = href.split('oobCode=')[1].split('&')[0]

            return internal_code, oob_code
    return None, None

def execute_get_request(usuario, dominios):
    url = "https://email-fake.com/"
    headers = {
        "Host": "email-fake.com",
        "Connection": "keep-alive",
        "Upgrade-Insecure-Requests": "1",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36",
        "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7",
        "Sec-Fetch-Site": "none",
        "Sec-Fetch-Mode": "navigate",
        "Sec-Fetch-User": "?1",
        "Sec-Fetch-Dest": "document",
        "sec-ch-ua": '"Not/A)Brand";v="8", "Chromium";v="126", "Google Chrome";v="126"',
        "sec-ch-ua-mobile": "?0",
        "sec-ch-ua-platform": "Windows",
        "Accept-Language": "es-ES,es;q=0.9",
        "Cookie": f'surl={dominios}%2F{usuario}',
        "Accept-Encoding": "gzip, deflate"
    }

    response = requests.get(url, headers=headers)

    # Uso de la función
    internal_code, oob_code = extract_codes_from_html(response.text)

    #print(response.text)

    # Extraer el código de verificación del contenido HTML
    verification_code = extract_verification_code(response.text)

    #if verification_code=="No Exit":
    #  proceso_completo()

    # Definir el patrón de búsqueda para delll
    patron = r"delll:\s*\"([^\"]+)\""

    # Aplicar la búsqueda utilizando regex
    resultado = re.search(patron, response.text)

    # Verificar si se encontró delll y obtener su valor
    if resultado:
        valor_delll = resultado.group(1)

    else:
        print("No se encontró delll en el código JavaScript.")


    return internal_code, str(verification_code).replace("Your confirmation code is ",""), valor_delll

    #return response.text




def proceso_completo():
    configurar_credenciales()
    email = os.environ.get("EMAIL_HEDRA")
    passwords = os.environ.get("PASS_HEDRA")
    # Paso 2: Obtener información de la sesión
    print("Obteniendo información de la sesión...")
    csrf_token, formatted_cookies = get_session_info()
    os.environ["CSRF_TOKEN"] = csrf_token
    os.environ["FORMATTED_COOKIE"] = formatted_cookies
    time.sleep(1)


    # Paso 3: Postear credenciales y obtener token de sesión
    print("Enviando credenciales...")
    session_token = post_credentials(email, passwords, formatted_cookies, csrf_token)
    if session_token:
        os.environ["SESSION_TOKEN"] = session_token
        print("Credenciales aceptadas.")

    time.sleep(5)

    usuario = os.environ.get("USER_HEDRA")
    dominio = os.environ.get("DOMAIN_HEDRA")

    # Paso 4: Buscar código interno
    print("Buscando código interno...")
    internal_code, oob_code, valor_delll = execute_get_request(usuario, dominio)


    time.sleep(5)
    email = os.environ.get("EMAIL_HEDRA")
    passwords = os.environ.get("PASS_HEDRA")
    session_token2 = os.environ.get("SESSION_TOKEN")
    # Paso 5: Verificar credenciales con el código
    print("Verificando credenciales con el código...")

    csrf_token1 = os.environ.get("CSRF_TOKEN")
    formatted_cookies1 = os.environ.get("FORMATTED_COOKIE")

    cookies = post_credentials_with_code(email, passwords, oob_code, formatted_cookies1, session_token2, csrf_token1)
    if cookies:
        print("Credenciales verificadas.")


    time.sleep(2)
    email = os.environ.get("EMAIL_HEDRA")
    passwords = os.environ.get("PASS_HEDRA")
    session_token2 = os.environ.get("SESSION_TOKEN")
    csrf_token1 = os.environ.get("CSRF_TOKEN")
    formatted_cookies1 = os.environ.get("FORMATTED_COOKIE")
    # Paso 6: Obtener nueva información de sesión
    print("Actualizando sesión...")
    session_token3 = get_session_info2(email, passwords, formatted_cookies1, session_token2)
    if session_token3:
        print("Sesión actualizada.")


    time.sleep(2)
    email = os.environ.get("EMAIL_HEDRA")
    passwords = os.environ.get("PASS_HEDRA")
    session_token2 = os.environ.get("SESSION_TOKEN")
    csrf_token1 = os.environ.get("CSRF_TOKEN")
    formatted_cookies1 = os.environ.get("FORMATTED_COOKIE")
    # Paso 7: Loguearse con los tokens obtenidos
    print("Iniciando sesión...")
    token_0, token_1 = post_sign_in(email, passwords, formatted_cookies1, session_token2, csrf_token1)
    if token_0 and token_1:
        print("Inicio de sesión exitoso.")


    # Paso 8: Obtener sesión con la URL deseada
    print("Obteniendo sesión...")
    formatted_cookies1 = os.environ.get("FORMATTED_COOKIE")
    status_code, access_token = get_session(formatted_cookies1, token_0, token_1)
    if access_token:
        os.environ["ACCESS_TOKEN_HEDRA"] = access_token


    print("Sesión establecida correctamente.")
    time.sleep(2)

    # Paso 9: Registrar usuario con el token de acceso
    print("Registrando usuario...")
    registrar_usuario(access_token)
    print("Proceso completo.")




def configurar_credenciales():
    print("Generando datos iniciales...")
    
    password_segug = generar_contrasena()
    response = enviar_formulario()
    sitio_domain = obtener_sitio_web_aleatorio(response.text)
    
    nombre_completo = generar_nombre_completo()
    email = f'{nombre_completo}@{sitio_domain}'
    passwords = password_segug

    usuario, dominio = email.split('@')

    os.environ["USER_HEDRA"] = usuario
    os.environ["DOMAIN_HEDRA"] = dominio

    os.environ["EMAIL_HEDRA"] = email
    os.environ["PASS_HEDRA"] = passwords



import random
import json
import requests
from tqdm.notebook import tqdm
import time
import hashlib
import base64
import os
from registro import *
from moviepy.video.io.VideoFileClip import VideoFileClip

def recortar_video(ruta_video):
    """
    Recorta un video eliminando una duración fija del inicio y del final,
    guardándolo con el mismo nombre y en la misma ruta, reemplazando el archivo original.
    
    Args:
        ruta_video (str): Ruta completa del archivo de video (entrada y salida).
    """
    # Duraciones fijas para recortar
    inicio_recorte = 0.13  # Recortar 13 milésimas de segundo del inicio
    final_recorte = 0.15  # Recortar 15 milésimas de segundo del final

    # Carga el video original
    clip = VideoFileClip(ruta_video)
    
    # Recorta el video
    duracion_total = clip.duration
    nuevo_clip = clip.subclip(inicio_recorte, duracion_total - final_recorte)
    
    # Genera un archivo temporal en la misma ruta
    temp_output_path = "temp_" + os.path.basename(ruta_video)
    nuevo_clip.write_videofile(temp_output_path, codec="libx264", audio_codec="aac")
    
    # Reemplaza el archivo original con el archivo temporal
    os.replace(temp_output_path, ruta_video)
    
    print(f"Video procesado y guardado en {ruta_video}")

def obteneravatar(access_token):
    url = "https://www.hedra.com/api/app/v1/app/projects"
    headers = {
        "Authorization": f"Bearer {access_token}",
        "sec-ch-ua": '"Not)A;Brand";v="99", "Google Chrome";v="127", "Chromium";v="127"',
        "sec-ch-ua-platform": '"Windows"',
        "sec-ch-ua-mobile": "?0",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, como Gecko) Chrome/127.0.0.0 Safari/537.36",
        "Content-Type": "application/json",
        "Accept": "*/*",
        "Sec-Fetch-Site": "same-origin",
        "Sec-Fetch-Mode": "cors",
        "Sec-Fetch-Dest": "empty",
        "Referer": "https://www.hedra.com/app/characters",
        "Accept-Language": "es-ES,es;q=0.9",
        "Accept-Encoding": "gzip, deflate",
    }

    response = requests.get(url, headers=headers)
    return response.json()

def eliminar_proyecto(joob_id, access_token):
    url = f"https://www.hedra.com/api/app/v1/app/projects/{joob_id}"
    headers = {
        "Authorization": f"Bearer {access_token}",
        "sec-ch-ua": '"Not)A;Brand";v="99", "Google Chrome";v="127", "Chromium";v="127"',
        "sec-ch-ua-platform": '"Windows"',
        "sec-ch-ua-mobile": "?0",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, como Gecko) Chrome/127.0.0.0 Safari/537.36",
        "Content-Type": "application/json",
        "Accept": "*/*",
        "Origin": "https://www.hedra.com",
        "Sec-Fetch-Site": "same-origin",
        "Sec-Fetch-Mode": "cors",
        "Sec-Fetch-Dest": "empty",
        "Referer": "https://www.hedra.com/app/characters",
        "Accept-Language": "es-ES,es;q=0.9",
        "Accept-Encoding": "gzip, deflate",
    }

    response = requests.delete(url, headers=headers)

    # Imprime el código de estado de la respuesta y el contenido de la respuesta
    #print(f"Status Code: {response.status_code}")
    #print(f"Response: {response.text}")



def video_to_base64(video_path):
    with open(video_path, "rb") as video_file:
        video_base64 = base64.b64encode(video_file.read()).decode('utf-8')
    return video_base64


# Función para actualizar la barra de progreso y descargar el video si el progreso es 100%
def update_progress_bar(current_progress, ruta_video, access_token, joob_ids):
    total_steps = 100
    video_url = None
    while current_progress < 1.0:
        # Simula obtener el progreso actualizado
        response = obteneravatar(access_token)
        #print("Proceso:", response)
        project = response['projects'][0]
        current_progress = project['progress']
        video_url = project['videoUrl']  # Obtener la URL del video
        #print("Video URL:", video_url)
        
        # Calcular el contador de progreso
        step = int(current_progress * total_steps)

        os.environ["PROGRESS_HEDRA"] = str(step)
        
        # Imprimir el progreso en la misma línea
        print(f"\rProgreso: {step}%", end='', flush=True)
        
        time.sleep(2)  # Ajusta el intervalo de actualización aquí

    if video_url:  # Si se ha obtenido una URL del video, descarga el video
        download_video(video_url, ruta_video, access_token, joob_ids)



# Función para generar un nombre de archivo seguro a partir de una URL
def generate_safe_filename(url):
    # Usa un hash MD5 de la URL para generar un nombre de archivo único y seguro
    hash_object = hashlib.md5(url.encode())
    return hash_object.hexdigest() + '.mp4'

# Función para descargar el video desde una URL
def download_video(url, ruta_video, access_token, joob_ids):
    #print("url",url)
    os.makedirs('/tmp/videos_tts', exist_ok=True)
    response = requests.get(url, stream=True)
    if response.status_code == 200:
        filename = generate_safe_filename(url)
        #print(url)
        #filename = "3424asdf.mp4"
        with open(f"{ruta_video}{filename}" , 'wb') as file:
            for chunk in response.iter_content(chunk_size=8192):
                if chunk:
                    file.write(chunk)
        print(f" Video downloaded as {filename}")


        # Ejecutar la función
        eliminar_proyecto(joob_ids, access_token)

        recortar_video(f"{ruta_video}{filename}")
        
        os.environ["VIDEO_PATH_HEDRA"] = f"{ruta_video}{filename}"

    else:
        print("Error downloading video")

def enviar_avatar_request(text, avatar_image, aspect_ratio, prompt, use_manual_seed, seed, voice_id, access_token):

    # Genera una semilla aleatoria si no se proporciona y si use_manual_seed es False
    if not use_manual_seed:
        seed = random.randint(1000000, 9999999)  # Genera un número aleatorio de 7 dígitos

    url = "https://www.hedra.com/api/app/v1/app/avatars/predict-async"
    headers = {
        "Host": "www.hedra.com",
        "Connection": "keep-alive",
        "Authorization": f"Bearer {access_token}",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/131.0.0.0 Safari/537.36",
        "sec-ch-ua": "\"Google Chrome\";v=\"131\", \"Chromium\";v=\"131\", \"Not_A Brand\";v=\"24\"",
        "Content-Type": "application/json",
        "Accept": "*/*",
        "Origin": "https://www.hedra.com",
        "Accept-Language": "es-ES,es;q=0.9",
        "Accept-Encoding": "gzip, deflate",
    }

    # Crear el cuerpo de la solicitud
    data = {
        "text": text,
        "avatar_image": avatar_image,
        "aspect_ratio": aspect_ratio,
        "avatar_image_input": {
            "prompt": prompt,
            "seed": seed
        },
        "audio_source": "tts",  # Usando TTS como la fuente de audio
        "voice_id": voice_id
    }

    # Realizar la solicitud POST
    response = requests.post(url, headers=headers, data=json.dumps(data))

    # Verificar la respuesta
    if response.status_code == 200:
        print("Request successful!")
        #print(response.json())  # Imprimir respuesta si es JSON
        # Supongamos que tienes el siguiente diccionario
        responses = response.json()

        # Extraer el job_id
        job_id = responses['job_id']
        return job_id
    else:
        proceso_completo()
        time.sleep(2)
        access = os.environ.get("ACCESS_TOKEN_HEDRA")
        #if access:
        procesar_avatar_request(voice_id,use_manual_seed,seed,aspect_ratio,prompt,text)

def procesar_avatar_request(
    selected_voice_origen_id,
    use_manual_seed,
    seed,
    aspect_ratio,
    prompt_txt,
    text
):

    
    access_token = os.environ.get("ACCESS_TOKEN_HEDRA")
    # Convertir la imagen en Base64 desde la ruta
    try:
        with open("/tmp/avatar_img.jpg", "rb") as img_file:
            base64_string = base64.b64encode(img_file.read()).decode("utf-8")
    except FileNotFoundError:
        raise FileNotFoundError("No se encontró la imagen en la ruta '/tmp/avatar_img.jpg'")

    # Validación de los parámetros
    if not base64_string:
        raise ValueError("Se debe proporcionar una cadena Base64 para la imagen del avatar")
    if not access_token:
        raise ValueError("Se debe proporcionar un token de acceso válido")

    # Enviar solicitud con los datos
    job_id = enviar_avatar_request(
        text=text,
        avatar_image=base64_string,
        aspect_ratio=aspect_ratio,
        prompt=prompt_txt,
        use_manual_seed=use_manual_seed,
        seed=seed,
        voice_id=selected_voice_origen_id,
        access_token=access_token,
    )

    # Procesar la respuesta
    if job_id:
        resultado = obteneravatar(access_token)
        #print("Resultado de la solicitud:", resultado)
        project = resultado['projects'][0]
        initial_progress = project['progress']
        update_progress_bar(initial_progress, "/tmp/videos_tts/", access_token, job_id)

